<h2>Documentation for Sequel (v<%= Sequel.version %>)</h2>

<h3>General Info, Guides, Examples, and Tutorials</h3>

<ul>
  <li>General Overviews<ul>
    <li><a href="rdoc/files/README_rdoc.html">README</a> (Introduction to Sequel, start here if new)</li>
    <li><a href="rdoc/files/doc/security_rdoc.html">Security</a></li>
    <li><a href="rdoc/files/doc/cheat_sheet_rdoc.html">Cheat Sheet</a></li>
    <li><a href="rdoc/files/doc/sql_rdoc.html">Sequel for SQL Users</a></li>
    <li><a href="rdoc/files/doc/object_model_rdoc.html">Sequel Object Model</a></li>
  </ul></li>
  <li>Databases<ul>
    <li><a href="rdoc/files/doc/opening_databases_rdoc.html">Connecting to a Database</a></li>
    <li><a href="rdoc/files/doc/transactions_rdoc.html">Database Transactions</a></li>
    <li><a href="rdoc/files/doc/schema_modification_rdoc.html">Schema Modification</a></li>
    <li><a href="rdoc/files/doc/migration_rdoc.html">Migrations</a></li>
    <li><a href="rdoc/files/doc/sharding_rdoc.html">Master/Slave Databases and Sharding</a></li>
    <li><a href="rdoc/files/doc/postgresql_rdoc.html">PostgreSQL Specific Support</a></li>
    <li><a href="rdoc/files/doc/mssql_stored_procedures_rdoc.html">Microsoft SQL Server Stored Procedure Support</a></li>
  </ul></li>
  <li>Datasets<ul>
    <li><a href="rdoc/files/doc/dataset_basics_rdoc.html">Dataset Basics</a></li>
    <li><a href="rdoc/files/doc/dataset_filtering_rdoc.html">Dataset Filtering</a></li>
    <li><a href="rdoc/files/doc/querying_rdoc.html">Querying in Sequel</a></li>
    <li><a href="rdoc/files/doc/prepared_statements_rdoc.html">Prepared Statements/Bound Variables</a></li>
    <li><a href="rdoc/files/doc/virtual_rows_rdoc.html">Virtual Row Blocks</a></li>
  </ul></li>
  <li>Models<ul>
    <li><a href="rdoc/files/doc/mass_assignment_rdoc.html">Model Mass Assignment</a></li>
    <li><a href="rdoc/files/doc/validations_rdoc.html">Model Validations</a></li>
    <li><a href="rdoc/files/doc/model_hooks_rdoc.html">Model Hooks</a></li>
    <li><a href="rdoc/files/doc/model_dataset_method_design_rdoc.html">Model Dataset Method Design</a></li>
    <li><a href="rdoc/files/doc/association_basics_rdoc.html">Model Associations Basics and Options</a></li>
    <li><a href="rdoc/files/doc/advanced_associations_rdoc.html">Advanced Model Associations</a></li>
    <li><a href="rdoc/files/doc/model_plugins_rdoc.html">Model Plugins</a></li>
  </ul></li>
  <li>Misc.<ul>
    <li><a href="rdoc/files/doc/code_order_rdoc.html">Code Order</a></li>
    <li><a href="rdoc/files/doc/reflection_rdoc.html">Reflection</a></li>
    <li><a href="rdoc/files/doc/thread_safety_rdoc.html">Thread Safety</a></li>
    <li><a href="rdoc/files/doc/testing_rdoc.html">Testing With Sequel</a></li>
    <li><a href="rdoc/files/doc/extensions_rdoc.html">Sequel Extensions</a></li>
    <li><a href="rdoc/files/doc/core_extensions_rdoc.html">Core Extensions</a></li>
    <li><a href="rdoc/files/doc/bin_sequel_rdoc.html">bin/sequel</a></li>
  </ul></li>
</ul>

<h3>RDoc</h3>

<ul>
  <li><a href="rdoc/index.html">Core/Model</a>, main pages:<ul>
    <li><a href="rdoc/classes/Sequel.html">Sequel</a> (main module)</li>
    <li><a href="rdoc/classes/Sequel/Database.html">Database</a></li>
    <li><a href="rdoc/classes/Sequel/Dataset.html">Dataset</a></li>
    <li><a href="rdoc/classes/Sequel/Model.html">Model</a></li>
    <li><a href="rdoc/classes/Sequel/Model/ClassMethods.html">Model Class Methods</a></li>
    <li><a href="rdoc/classes/Sequel/Model/InstanceMethods.html">Model Instance Methods</a></li>
    <li><a href="rdoc/classes/Sequel/Model/Associations/ClassMethods.html">Model Association Class Methods</a></li>
    <li><a href="rdoc/classes/Sequel/Model/Associations/DatasetMethods.html">Model Eager Loading Methods</a></li>
</ul></li>
  <li><a href="rdoc-adapters/index.html">Adapters</a></li>
  <li><a href="rdoc-plugins/index.html">Extensions/Plugins</a></li>
</ul>

<h3><a href="rdoc/files/CHANGELOG.html">Change Log</a></h3>

<h3>Release Notes</h3>
<ul>
  <% %w'5 4 3'.each do |i| %>
    <li>Sequel <%= i %>:<ul>
      <% 
         lines = []
         Dir["../doc/release_notes/#{i}.*.txt"].map{|f| File.basename(f)}.each do |f|
           (lines[f.split('.')[1].to_i/10] ||= []) << f
         end
         lines.reverse.each do |fs|
      %>
        <li>
          <% fs.sort_by{|f| f.split('.').map{|x| x.to_i}}.reverse.each do |f| %>
            <a href="rdoc/files/doc/release_notes/<%= f.gsub('.', '_') %>.html"><%= f.sub(/\.txt$/, '').sub(/(..)\.0$/, '\\1') %></a> |
          <% end %>
        </li>
      <% end %>
    </ul></li>
  <% end %>
  <% %w'2 1'.each do |i| %>
    <li>Sequel <%= i %>: 
      <% Dir["../doc/release_notes/#{i}.*.txt"].map{|f| File.basename(f)}.sort_by{|f| f.split('.').map{|x| x.to_i}}.reverse.each do |f| %>
        <a href="rdoc/files/doc/release_notes/<%= f.gsub('.', '_') %>.html"><%= f.sub(/\.txt$/, '').sub(/(..)\.0$/, '\\1') %></a> |
      <% end %>
    </li>
  <% end %>
</ul>

<h3><a href="rdoc/files/MIT-LICENSE.html">License</a></h3>

<h3>Presentations</h3>

<ul>
<li><a href="http://code.jeremyevans.net/presentations/rubyc2014-2/index.html">&quot;Give-and-Go with PostgreSQL and Sequel&quot; Presentation at RubyC 2014</a> (<a href="http://www.youtube.com/watch?v=toAcnwqlU1Q">Video</a>)</li>
<li><a href="http://code.jeremyevans.net/presentations/rubyc2014-1/index.html">&quot;Deep Dive Into Eager Loading Limited Associations&quot; Presentation at RubyC 2014</a> (<a href="http://www.youtube.com/watch?v=KbF7RMk_2Qo">Video</a>)</li>
<li><a href="http://code.jeremyevans.net/presentations/railsclub2013/index.html?trans=no">&quot;Give-and-Go with PostgreSQL and Sequel&quot; Presentation at RailsClub 2013</a> (<a href="http://code.jeremyevans.net/presentations/railsclub2013/index.html">In Russian</a>) (<a href="http://live.digicast.ru/ru/view/2116">Video</a>, starts about 6:30)</li>
<li><a href="http://code.jeremyevans.net/presentations/heroku201205/index.html">&quot;The Development of Sequel&quot; Presentation in May 2012 at Heroku</a></li>
<li><a href="http://code.jeremyevans.net/presentations/pgwest2011/index.html">&quot;Sequel: The Database Toolkit for Ruby&quot; Presentation at PostgreSQL Conference West 2011</a></li>
<li><a href="http://code.jeremyevans.net/presentations/lsrc2009_presentation/sequel-lsrc2009.html">&quot;Sequel: SQL in Ruby&quot; Presentation at Lone Star Ruby Conference 2009</a></li>
<li><a href="http://code.jeremyevans.net/presentations/rk2009_presentation/sequel-rubykaigi2009.html">&quot;Sequel: SQL in Ruby&quot; Presentation at RubyKaigi 2009</a></li>
<li><a href="http://code.jeremyevans.net/presentations/larc2009_presentation/sequel-larc2009-pres.html">&quot;Sequel: The Database Toolkit for Ruby&quot; Presentation at LA Ruby Conf 2009</a> (<a href="http://confreaks.com/videos/246-larubyconf2009-sequel">Video</a>)</li>
<li><a href="http://code.jeremyevans.net/presentations/mwrc2009_presentation.html">&quot;Sequel: The Database Toolkit for Ruby&quot; Presentation at MountainWest RubyConf 2009</a> (<a href="http://confreaks.com/videos/51-mwrc2009-sequel">Video</a>) (<a href="http://code.jeremyevans.net/presentations/mwrc2009_presentation.txt">Transcript</a>)</li>
</ul>
